//css_ref System.Data;
//css_ref System.Transactions;
//css_ref System.Xml;
//css_ref Interop.SAPbobsCOM.dll;
//css_ref log4net.dll;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;
//css_ref Produmex.Sbo.Logex.Paris8.dll;

using System;
using System.Diagnostics;
using System.Reflection;
using System.Transactions;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Attributes;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.Resources;
using Produmex.Sbo.Logex.Data;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.Providers;

using Produmex.Sbo.Logex.Paris8.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Paris8.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Paris8.Generators.PickListRobot;
using static Produmex.Foundation.ProdumexValues;

namespace AddUpdateScript
{
	class Program
    {
        static string CONNECTION_STRING = "Server=jankisz;CompanyDB=Trunk_PMX;ExtraDb=PMX_Trunk_PMX;UseTrusted=False;DbUserName=sa;DbPassword=B1Admin;UserName=manager;Password=manager;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2014;DBCultureInfo=en-US;LicenseServer=localhost:30000";


        /// <summary>
        /// Used for logging
        /// </summary>
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        static void Main(string[] args)
        {
            try
            {
                LogProvider.SetConsoleAppender("WARN");       
                Execute();
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
            }
            if (Debugger.IsAttached)
            {
                Console.WriteLine("Press <Enter>");
                Console.ReadLine();
            }
        }

		private static readonly Assembly s_paris8Assembly = typeof(Produmex.Sbo.Logex.Paris8.SboCommands).Assembly;
		private static string LOCATION_SCRIPTS = "Produmex.Sbo.Logex.Paris8.Scripts.";

		static void Execute()
        {
            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (SboConnection conn = new SboConnection(CONNECTION_STRING))
                {
                    conn.Open(); //Open connection to SAP
                    Console.WriteLine("Connection is open to database");

					const string FORMTYPE = "PAR_OPOL";
					const string QUALITYSTATUS_ITEMID = "EDT_QYS";
					const string WAREHOUSE_ITEMID = "EDT_WH";
					const string WAREHOUSECOLUMN_COLUMNID = "WhsCode";
					const string STORAGELOCATION_ITEMID = "EDT_SL";
					const string STORAGELOCATIONCOLUMN_ITEMID = "MTX_OPOL";
					const string STORAGELOCATIONCOLUMN_COLUMNID = "StorLocCod";
					const string USERQUERY_LOCATIONSFORWAREHOUSE = "PAR_LOCATIONS_FOR_WAREHOUSE";
					const string USERQUERY_LOCATIONSFORWAREHOUSECOLUMN = "PAR_LOCATIONS_FOR_WAREHOUSE_COLUMN";

					PmxDbVersionUpdaterProvider dbvup = new PmxDbVersionUpdaterProvider(conn);

					SboUserQueryCategoryProvider uqcProvider = new SboUserQueryCategoryProvider(conn);
                 

                    int categoryCode;
					using (SboUserQueryCategory category = uqcProvider.GetUserQueryCategoryByName(PmxUserQueryCategoryName.PMX_ADDON))
					{
						categoryCode = category.Code;
					}

					int? userQueryID = dbvup.GetUserQuery(PmxUserQueryName.QUALITY_STATUSES);

					if (userQueryID.HasValue)
					{
						dbvup.AddUpdateFormattedSearch(
							FORMTYPE,
							QUALITYSTATUS_ITEMID,
							null,
							userQueryID.Value,
							null,
							false, false);
					}

					int userQueryIDLocations = dbvup.AddUpdateUserQuery(USERQUERY_LOCATIONSFORWAREHOUSE, categoryCode,
						SboCommands.GetQuerySelectLocationForWarehouseGlobal("$[$EDT_WH.0.0]", conn.DbCultureInfo, conn.DbTool));

					dbvup.AddUpdateFormattedSearch(
						FORMTYPE,
						STORAGELOCATION_ITEMID,
						null,
						userQueryIDLocations,
						WAREHOUSE_ITEMID,
						false, false);

					userQueryIDLocations = dbvup.AddUpdateUserQuery(USERQUERY_LOCATIONSFORWAREHOUSECOLUMN, categoryCode,
						SboCommands.GetQuerySelectLocationForWarehouseGlobal("$[$MTX_OPOL.WhsCode.0]", conn.DbCultureInfo, conn.DbTool));

					dbvup.AddUpdateFormattedSearch(
						FORMTYPE,
						STORAGELOCATIONCOLUMN_ITEMID,
						STORAGELOCATIONCOLUMN_COLUMNID,
						userQueryIDLocations,
						WAREHOUSECOLUMN_COLUMNID,
						false, false);



                    //Pick list robot                    
                    dbvup.AddUpdateExtension(PickListRobot.IDs.ExtCode, "Paris8 Pick List robot",
                 typeof(Produmex.Sbo.Logex.Paris8.Generators.PickListRobot.PickListRobot), Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionTypeCodes.IPICKLISTROBOT);

                    dbvup.AddUpdateExtensionParameter(
                      PickListRobot.IDs.ExtParamMaxLines,
                      "Max. number of lines per pick list?",
                      PickListRobot.IDs.ExtCode,
                      Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.PmxExtensionCodes.SYSTEM_INT32_CONVERTOR,
                      "0",
                      PmxExtensionParameterCodesPAR.PICKLIST_ROBOT_MAX_lINES,
					  General.Grouping.General
					  );


                    // * UDT's *                  
                    dbvup.AddUpdateUserDefinedTable(typeof(PmxPickListRobotRequirements));
                    dbvup.AddUpdateUserDefinedObject(typeof(PmxPickListRobotRequirements));


                }

                //Complete transaction
                scope.Complete();
            }


          

            //PMX connection
            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (PmxDbConnectionSql dbConn = new PmxDbConnectionSql(CONNECTION_STRING))
                {
                    dbConn.Open();

                    Console.WriteLine("Tables");
                  

                    Console.WriteLine("Views");

					dbConn.DbTool.CreateUpdateView("PAR_OPEN_PURCHASE_ORDER_LINES", ResourceHelper.ReadResourceContentAsString(s_paris8Assembly, LOCATION_SCRIPTS + "Views.PAR_OPEN_PURCHASE_ORDER_LINES.sql"));
                    dbConn.DbTool.CreateUpdateView("PMX_PAR_OPEN_WHS_TRANSFER_LINES", ResourceHelper.ReadResourceContentAsString(s_paris8Assembly, LOCATION_SCRIPTS + "Views.PMX_PAR_OPEN_WHS_TRANSFER_LINES.sql"));
	                dbConn.DbTool.CreateUpdateView("PMX_PAR_PROPOSALS_TO_PICK_LIST", ResourceHelper.ReadResourceContentAsString(s_paris8Assembly, LOCATION_SCRIPTS + "Views.PMX_PAR_PROPOSALS_TO_PICK_LIST.sql"));
	                dbConn.DbTool.CreateUpdateView("PMX_P8_PICK_LISTS_TO_CLOSE", ResourceHelper.ReadResourceContentAsString(s_paris8Assembly, LOCATION_SCRIPTS + "Views.PMX_P8_PICK_LISTS_TO_CLOSE.sql"));
	                dbConn.DbTool.CreateUpdateView("PMX_P8_WHS_TRANSFER_REQUESTS_TO_CLOSE", ResourceHelper.ReadResourceContentAsString(s_paris8Assembly, LOCATION_SCRIPTS + "Views.PMX_P8_WHS_TRANSFER_REQUESTS_TO_CLOSE.sql"));

					//Complete transaction
					scope.Complete();
                }
            }

			Console.WriteLine("Done ...");
        }
    }
}
