
using System;
using System.Reflection;
using System.Transactions;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Attributes;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.Resources;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.Providers;
using System.Diagnostics;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using System.Text;
using Produmex.Foundation.Data.SqlClient;
using System.Globalization;

//css_ref System.Data;
//css_ref System.Xml;
//css_ref Interop.SAPbobsCOM.dll;
//css_ref log4net.dll;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;






namespace AddUpdateData
{
	class Program
	{

		static string CONNECTION_STRING = "Server=jankisz;CompanyDB=Trunk_PMX;ExtraDb=PMX_Trunk_PMX;UseTrusted=False;DbUserName=sa;DbPassword=B1Admin;UserName=manager;Password=manager;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2014;DBCultureInfo=en-US;LicenseServer=localhost:30000";


		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		static void Main(string[] args)
		{
			try
			{
				ClosePicklistsAndWarehouseTransfers();
			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				Console.WriteLine(ex);
				Console.ReadLine();
			}

		}





		private static void ClosePicklistsAndWarehouseTransfers()
		{
			using (SboConnection conn = new SboConnection(CONNECTION_STRING))
			{
				conn.Open();
				Console.Write("Connection is open");

				string query = @"SELECT * FROM PMX_P8_PICK_LISTS_TO_CLOSE";

				using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, conn))
				{
					s_log.Info($"Found {rs.RecordCount} open picklist(s)");

					PmxPickListProvider plProv = new PmxPickListProvider(conn);

					while (!rs.EoF)
					{
						int plDocEntry = rs.GetTypedValue<int>("DocEntry");

						s_log.Info($"Closing picklist {plDocEntry}");

						try
						{
							using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
							{
								conn.EnlistTransaction(Transaction.Current);
								PmxPickList pl = plProv.GetBO(plDocEntry);
								plProv.CloseDocument(pl);
								scope.Complete();
							}

							s_log.Info($"Closed picklist {plDocEntry}");
						}
						catch (Exception ex)
						{
							s_log.Error(ex);
						}

						rs.MoveNext();
					}
				}


				query = @"SELECT * FROM PMX_P8_WHS_TRANSFER_REQUESTS_TO_CLOSE";

				using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, conn))
				{
					s_log.Info($"Found {rs.RecordCount} open warehouse transfer(s)");

					PmxPickListProvider plProv = new PmxPickListProvider(conn);
					PmxPickListProposalProvider plpProv = new PmxPickListProposalProvider(conn);
					SboStockTransferRequestProvider stProv = new SboStockTransferRequestProvider(conn);

					while (!rs.EoF)
					{
						bool shouldCloseTransfer = true;
						int? plDocEntry = rs.GetTypedValue<int?>("PicklistDocEntry");
						int? plpDocEntry = rs.GetTypedValue<int?>("PicklistProposalDocEntry");
						int whTransferDocEntry = rs.GetTypedValue<int>("TransferDocEntry");

						s_log.Info($"Open warehouse transfer {whTransferDocEntry} found with picklist proposal {plpDocEntry} and picklist {plDocEntry}");

						if (plDocEntry.HasValue && plDocEntry.Value > 0)
						{
							s_log.Info($"Closing picklist {plDocEntry}");

							try
							{
								using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
								{
									conn.EnlistTransaction(Transaction.Current);
									PmxPickList pl = plProv.GetBO(plDocEntry.Value);
									plProv.CloseDocument(pl);

									scope.Complete();
                                }
                                s_log.Info($"Closed picklist {plDocEntry}");
							}
                            catch (Exception ex)
							{
								s_log.Error(ex);
								shouldCloseTransfer = false;
                                s_log.Info($"Could not close picklist {plDocEntry}");
                            }
						}
						else if (plpDocEntry.HasValue && plpDocEntry.Value > 0)
                        {
                            s_log.Info($"Closing picklist proposal {plpDocEntry}");

                            try
                            {
                                using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
                                {
                                    conn.EnlistTransaction(Transaction.Current);
                                    PmxPickListProposal plp = plpProv.GetBO(plpDocEntry.Value);
                                    plpProv.CloseDocument(plp);

                                    scope.Complete();
                                }

                                s_log.Info($"Closed picklist proposal {plpDocEntry}");
                            }
                            catch (Exception ex)
                            {
                                s_log.Error(ex);
                                shouldCloseTransfer = false;
                                s_log.Info($"Could not close picklist proposal {plpDocEntry}");
                            }
                        }

                        s_log.Info($"Closing warehouse transfer {whTransferDocEntry}");

						if (shouldCloseTransfer)
						{
							try
							{
								using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
								{
									conn.EnlistTransaction(Transaction.Current);
									SboStockTransferRequest st = stProv.GetBO(whTransferDocEntry);
									stProv.CloseBO(st);

									scope.Complete();
								}

                                s_log.Info($"Closed warehouse transfer {whTransferDocEntry}");
                            }
                            catch (Exception ex)
							{
								s_log.Error(ex);
                                s_log.Info($"Could not close warehouse transfer {whTransferDocEntry}");
                            }
						}

						rs.MoveNext();
					}
				}
			}
		}
	}
}